// Библиотека для работы с протоколом I2C (порты A5/SCL и A4/SDA)
//Подключаем библиотеку I2C.
//!!!  Если бибилиотеки нет Установите её из архива Wire.zip
#include <Wire.h>
#include <IRremote.h>
// Порт для IR-приемника.
int RECV_PIN = 16;
// Создание IR-приемника.
IRrecv irrecv(RECV_PIN);
//Переменная для результатов IR-приемника.
decode_results results;
int PROG;
const int MPU_addr = 0x68; // упрощеный I2C адрес нашего гироскопа/акселерометра MPU-6050.
// переменные для хранения данных возвращаемых прибором.
int16_t AcX, AcY, AcZ, Tmp, GyX, GyY, GyZ;
double AcYsum, GyXsum;
// Угол поворота вокруг оси Z.
float Ang_;
const int key_flash = 14;
const int flash_data = 15;
// Объявляем переменные для хранения состояния двух моторов.
int motor_L1, motor_L2, input_L;
int motor_R1, motor_R2, input_R;
// Пин информационного светодиода.
int  Line_L = 14;
float CompensatorZ, CompensatorAcX;
//смещение нуля баланса.
double balancing_zerro = 2.4;
//Время на разгон/торможение в микросекундах.
const long time_razgon = 100000;
const int max_speed = 255;
const int min_speed = 255;
//=======================================================

void Data_mpu6050()
{
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x3B);  //Готовим для чтения регистры  с адреса 0x3B.
  Wire.endTransmission(false);
  // Запрос 14 регистров.
  Wire.requestFrom(MPU_addr, 14, true);
  // 0x3B (ACCEL_XOUT_H) & 0x3C (ACCEL_XOUT_L)
  AcX = Wire.read() << 8 | Wire.read();
  // 0x3D (ACCEL_YOUT_H) & 0x3E (ACCEL_YOUT_L)
  AcY = Wire.read() << 8 | Wire.read();
  // 0x3F (ACCEL_ZOUT_H) & 0x40 (ACCEL_ZOUT_L)
  AcZ = Wire.read() << 8 | Wire.read();
  // 0x41 (TEMP_OUT_H) & 0x42 (TEMP_OUT_L)
  Tmp = Wire.read() << 8 | Wire.read();
  // 0x43 (GYRO_XOUT_H) & 0x44 (GYRO_XOUT_L)
  GyX = Wire.read() << 8 | Wire.read();
  // 0x45 (GYRO_YOUT_H) & 0x46 (GYRO_YOUT_L)
  GyY = Wire.read() << 8 | Wire.read();
  // 0x47 (GYRO_ZOUT_H) & 0x48 (GYRO_ZOUT_L)
  GyZ = Wire.read() << 8 | Wire.read();
}
// задержка но с расчетом изменения Z угла.
void time_gyro(float mill_sec)
{
  if
  long ms = mill_sec;
  unsigned long timer = micros();
  unsigned long endtime = micros() + long(ms * 1000.0);
  while (endtime > timer) {
    timer = micros();
    Data_mpu6050();
    Ang_ = Ang_ + (((float)(GyZ) - CompensatorZ) * (float)(micros() - timer)) / 131000000.0;
    delayMicroseconds(1);
  }
}
//==========================================================

// Функция инициализации управления моторами.
void setup_motor_system(int L1, int  L2, int iL, int  R1, int R2, int iR)
{
  // Заносим в переменные номера контактов (пинов) Arduino.
  motor_L1 = L1; motor_L2 = L2; input_L = iL;
  // Для левых и правых моторов машинки.
  motor_R1 = R1; motor_R2 = R2; input_R = iR;
  // Переводим указанные порты в состояние вывода данных.
  pinMode(motor_L1, OUTPUT);
  pinMode(motor_L2, OUTPUT);
  pinMode(input_L, OUTPUT);
  pinMode(motor_R1, OUTPUT);
  pinMode(motor_R2, OUTPUT);
  pinMode(input_R, OUTPUT);
}
// Функция задает скорость двигателей.
void setspeed(int LeftSpeed, int RightSpeed)
{
  // Задаем ширину положительного фронта от 0 до 255.
  analogWrite(input_L, LeftSpeed);
  analogWrite(input_R, RightSpeed);
  // Чем больше, тем интенсивнее работает мотор.
}
// движение вперед.
void forward()
{
  // Если двигатель будет работать не в ту сторону,
  // поменять на нем контакты местами.
  digitalWrite(motor_L1, HIGH);
  digitalWrite(motor_L2, LOW);
  digitalWrite(motor_R1, HIGH);
  digitalWrite(motor_R2, LOW);
}
// Поворот налево с блокировкой левых колес.
void forward_left()
{
  // блокируем вращение левых колес.
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_L2, LOW);
  // правые колеса вращаются.
  digitalWrite(motor_R1, HIGH);
  digitalWrite(motor_R2, LOW);
}
// Поворот налево.
void left()
{
  // левые колеса вращаются назад
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_L2, HIGH);
  // правые колеса вращаются.
  digitalWrite(motor_R1, HIGH);
  digitalWrite(motor_R2, LOW);
}
// Поворот направо с блокировкой правых колес.
void forward_right()
{
  // левые колеса вращаются.
  digitalWrite(motor_L1, HIGH);
  digitalWrite(motor_L2, LOW);
  // блокируем вращение правых колес.
  digitalWrite(motor_R1, LOW);
  digitalWrite(motor_R2, LOW);
}
void right()
{
  // левые колеса вращаются.
  digitalWrite(motor_L1, HIGH);
  digitalWrite(motor_L2, LOW);
  //правые колеса вращаются назад.
  digitalWrite(motor_R1, LOW);
  digitalWrite(motor_R2, HIGH);
}
// Включаем движение назад.
void backward()
{
  // Меняем направление вращения двигателей.
  digitalWrite(motor_L2, HIGH);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, HIGH);
  digitalWrite(motor_R1, LOW);
}
void backward_left()
{
  // Меняем направление вращения двигателей.
  digitalWrite(motor_L2, LOW);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, HIGH);
  digitalWrite(motor_R1, LOW);
}
void backward_right()
{
  // Меняем направление вращения двигателей.
  digitalWrite(motor_L2, HIGH);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, LOW);
  digitalWrite(motor_R1, LOW);
}
void _stop()
{
  // Блокируем все колеса.
  digitalWrite(motor_L2, LOW);
  digitalWrite(motor_L1, LOW);
  digitalWrite(motor_R2, LOW);
  digitalWrite(motor_R1, LOW);
}
//======================================
// Ехать вперед заданное время. ////////
//======================================
void forward_t( long microsec)
{
  int _speed = min_speed;
  bool flag = false;
  microsec = microsec + micros();
  long microsec_begin_tormog = microsec - time_razgon;
  float Ang_2 = Ang_; long timer;
  do {
    timer = micros();
    if (Ang_ < (-1.0 + Ang_2)) {
      forward_left(); time_gyro(0.5);
    }
    else if (Ang_ > (1.0 + Ang_2)) {
      forward_right(); time_gyro(0.5);
    }
    else {
      forward(); time_gyro(0.5); _stop(); time_gyro(0.8);
    }
    if ((timer > microsec_begin_tormog)) {
      _stop(); time_gyro(0.5);
    }
  } while (microsec > timer);
  _stop();
}
//======================================
// Ехать назад заданное время. ////////
//======================================
void backward_t( long microsec)
{
  int _speed = min_speed;
  bool flag = false;
  microsec = microsec + micros();
  long microsec_begin_tormog = microsec - time_razgon;
  float Ang_2 = Ang_; long timer;
  do {
    timer = micros();
    if (Ang_ < (-1.0 + Ang_2)) {
      backward_right(); time_gyro(0.5);
    }
    else if (Ang_ > (1.0 + Ang_2)) {
      backward_left(); time_gyro(0.5);
    }
    else {
      backward(); time_gyro(0.5);
    }
    if ((timer > microsec_begin_tormog)) {
      _stop(); time_gyro(0.5);
    }
  } while (microsec > timer);
  _stop();
}
//======================================
// Поворот на заданный угол. ////////
//======================================
void Angle(float ang)
{
  int _speed = max_speed;
  bool flag = false;
  float Ang_2 = Ang_; long timer;
  setspeed(_speed, _speed);
  //возможно стоит уменьшить скорость, если угол маленький.
  //if (abs(ang) < 8) setspeed(230, 230);
  if (ang == 0) return;
  if (ang < 0)
  {
    do {
      right(); time_gyro(1); _stop(); time_gyro(1);
    } while (Ang_ > (ang + Ang_2));
  }
  else
  {
    do {
      left(); time_gyro(1); _stop(); time_gyro(1);
    } while (Ang_ < (ang + Ang_2));
  }
  _stop();
  Ang_2 = Ang_ - (ang + Ang_2);
  if (abs(Ang_2) > 1.0)
    Angle(-Ang_2);
  setspeed(max_speed, max_speed);
}

//Держать угол определенное время
void Angle_t( long microsec)
{
  //time_razgon
  int _speed = min_speed;
  bool flag = false;
  microsec = microsec + micros();
  long microsec_begin_tormog = microsec - time_razgon;
  float Ang_t = Ang_; long timer;
  setspeed(_speed, _speed);
  _stop();
  do {

    timer = micros();

    if (Ang_ < (Ang_t - 1.0))   {
      left();   time_gyro(1);   _stop();
    }
    else if (Ang_ > (Ang_t + 1.0)) {
      right();       time_gyro(1);       _stop();
    } else   _stop(); time_gyro(1);
  } while (microsec > timer);
  _stop(); time_gyro(10);
}
//=========================================================
//================================================
void setup()
{
  // Заносим в переменные номера контактов (пинов) Arduino.
  // Для левых и правых моторов машинки.
  // Двигатели остановлены.
  Wire.begin();
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x6B);  // Производим запись в регистр энергосбережения MPU-6050
  Wire.write(0);     // устанавливем его в ноль
  Wire.endTransmission(true);
  //Serial.begin(9600);
  // pinMode(s_diod, OUTPUT);
  AcYsum = 0;
  GyXsum = 0;
  //delay(1000);
  CompensatorZ = 0; CompensatorAcX = 0;
  for (int i = 0; i < 500; i++)
  {
    delay(10);
    Data_mpu6050();
    CompensatorZ += GyZ;
    CompensatorAcX += AcX;
  }
  CompensatorZ = CompensatorZ / 500.0;
  CompensatorAcX = CompensatorAcX / 500.0;
  //  Serial.print("compensatorZ=");
  //  Serial.println(CompensatorZ);
  //  digitalWrite(s_diod, HIGH);
  // Инициализируем сервомотор, управление 9 портом.
  //  neck.attach(9);
  // Заносим в переменные номера контактов (пинов) Arduino.
  // Для левых и правых моторов машинки.
  pinMode(key_flash, OUTPUT);
  setup_motor_system(3, 4, 11, 7, 8, 10);
  // Двигатели остановлены.
  setspeed(255, 255);
  irrecv.enableIRIn();
}
//=========================================================
// Основная программа.
void loop()
{
  int high_, low_; float ang;
  long micro, hod = 700000;
  if (irrecv.decode(&results))
  {
    switch (results.value) {
      // 1
      case 0xFF6897:
        Angle(10);
        break;
      // 2
      case 0xFF9867:
        Angle(20);
        break;
      // 3
      case 0xFFB04F:
        Angle(30);
        break;
      // 4
      case 0xFF30CF:
        Angle(40);
        break;
      // 5
      case 0xFF18E7:
        Angle(50);
        break;
      // 6
      case 0xFF7A85:
        Angle(60);
        break;
      // Вперед 10 сек
      case 0xFF629D:
        forward_t(10000000);
        break;
      // Назад 10 сек
      case 0xFFA857:
        backward_t(10000000);
        break;
      case 0xFF02FD:
        Angle_t(100000000);
        break;
    }
    irrecv.resume();
  }
  delay(10);
}

